/**
 * @file
 * Provides GridStack admin utilities for Layout Builder inline.
 *
 * Cheaper alt than full-blown class overrides via hook_element_plugin_alter
 * aside from the fact other modules might also race overriding that class.
 */

(function ($, Drupal, _db, GridStyle, _doc) {

  'use strict';

  var _body = _doc.body;
  var _gsActionAjax = 'gridstack__action--ajax-styles';
  var _gsActionJs = 'gridstack__action--js-styles';
  var _lbLinkConfigClass = 'layout-builder__link--configure';
  var _lbLinkConfigSelector = '.' + _lbLinkConfigClass;
  var _lbHighligtedClass = 'is-layout-builder-highlighted';
  var _lbSectionSelector = '.layout-builder__section';
  var _dataGsSection = 'data-gs-section';
  var _gsSection = '.layout-builder__section[data-gs-section]';
  var _gsRegionActive = 'is-gs-region-active';
  var _gsRoot = '.is-gs-lb';
  var _gsRootActivated = _gsRoot + '.' + _lbHighligtedClass;
  var _dataGsRegion = 'data-gs-region';
  var _formSubmitActive = 'form-submit--gs-active';
  var _offcanvas = '#drupal-off-canvas';
  var idLbTopLink = 'gsLbTopLink';
  var elLbTopLink = _lbLinkConfigSelector + ':not(.' + _gsActionAjax + ')';
  var idVariantEdit = 'gsVariantEdit';
  var elVariantEdit = '.btn--editor';
  var idContainer = 'gsContainer';
  var elContainer = '.is-gs-lb';
  var idLabelVariant = 'gsLabelVariant';
  var elLabelVariant = '.form-text--label-variant';
  var _gridstackAction = 'gridstack__action';

  /**
   * Prepare region markers.
   *
   * @param {jQuery.Event} e
   *   The mousedown event
   */
  function gsLbTopLink(e) {
    gsCleanOut();
    gsSectionActivate(e);
  }

  /**
   * Cleans up region markers.
   *
   * @param {jQuery.Event} e
   *   The mousedown event
   */
  function gsModalClose(e) {
    // dialog:afterclose is called after ML AJAX upload, too.
    // dialog:beforeclose is called when the form is not save.
    window.setTimeout(function () {
      if (!$(_offcanvas).length) {
        gsCleanOut();
      }
    }, 3000);
  }

  /**
   * Cleans up region markers.
   */
  function gsCleanOut() {
    var $gsSection = $(_gsSection);
    var $gsRegionActive = $('.' + _gsRegionActive);

    $('body').removeAttr(_dataGsRegion);

    if ($gsSection.length) {
      $(_gsSection).removeAttr(_dataGsSection);
    }

    if ($gsRegionActive.length) {
      $gsRegionActive.removeClass(_gsRegionActive).attr('id', '');
      $gsRegionActive.each(function (i, item) {
        $(item).removeClass(_gsRegionActive).attr('id', '');
      });
    }

    $('.' + _formSubmitActive).removeClass(_formSubmitActive);
  }

  /**
   * Prepares region markers.

   * @param {jQuery.Event} e
   *   The click/ mousedown event.
   */
  function gsSectionActivate(e) {
    var $gsSection = $(_gsSection);
    var $link = $(e.target);

    if (!$gsSection.length) {
      $gsSection = $link.closest(_lbSectionSelector);
    }

    if (!$(_gsSection).length) {
      $gsSection = $(_gsRootActivated).closest(_lbSectionSelector);
    }

    if ($gsSection.length) {
      var $gridstack = $(_gsRootActivated);

      if ($gridstack.length) {
        $('body').attr(_dataGsRegion, true);
        $gsSection.attr(_dataGsSection, true);
      }
    }
  }

  /**
   * Reacts on clicking gridstack__action--ajax-styles link.
   *
   * @param {jQuery.Event} e
   *   The mousedown event.
   */
  function gsRegionActivate(e) {
    var $link = $(e.target);
    var regionId = $link.parent().data('gsRegion');

    gsCleanOut();
    gsSectionActivate(e);

    $link.closest('.gridstack__box').addClass(_gsRegionActive).attr('id', _gsRegionActive);
    $('body').attr(_dataGsRegion, regionId);
  }

  /**
   * GridStack functions.
   *
   * @param {HTMLElement} $element
   *   The modal HTML element.
   * @param {String} type
   *   The current event type.
   */
  function showActiveRegion($element, type) {
    var activeRid = $('body').attr(_dataGsRegion);
    if (activeRid !== 'undefined') {
      var activeRegion = '[data-gs-region="' + activeRid + '"]';
      var $activeRegion = $element.find(activeRegion);
      if ($activeRegion.length) {
        var container = $activeRegion.data('gsRegionContainer');
        var $container = $element.find('[data-gs-region="' + container + '"]');

        $activeRegion.attr('open', true).removeClass('visually-hidden');

        $('.is-gs-main-settings', $element).attr('open', false);
        $element.find('[data-gs-region]:not(' + activeRegion + ', .is-gs-main-settings)').addClass('visually-hidden');
        $element.find('.is-gs-aside').addClass('visually-hidden');

        if ($container.length) {
          $container.attr('open', false).removeClass('visually-hidden');
        }
      }
    }
  }

  $(window).on('dialog:aftercreate', function (e, dialog, $element, settings) {
    showActiveRegion($element, 'dialog:aftercreate');
  });

  $(window).on('dialog:beforeclose dialog:afterclose', gsModalClose);

  /**
   * Reacts on button `Edit variant` AJAX events.
   *
   * @param {jQuery.Event} e
   *   The event triggered by a `mousedown` event.
   */
  function gsVariantEdit(e) {
    var $btn = $(e.target);

    $btn.closest(_lbSectionSelector).addClass('is-gs-editor-active');
  }

  /**
   * GridStack functions.
   *
   * @param {HTMLElement} container
   *   The .is-gs-lb HTML element.
   */
  function gsContainer(container) {
    var $container = $(container);
    var $lbSection = $container.closest(_lbSectionSelector);
    var $lbConfig = $('>' + _lbLinkConfigSelector, $lbSection);

    // In case we have another wrapper (.gridstack-wrapper) above .gridstack.
    // We didn't hard-code class name previously since they might change.
    // This time is a willy-nilly. Also to support LB_UX.
    if (!$lbConfig.length) {
      // layout-builder__actions.
      $lbConfig = $lbSection.find('> div:first ' + _lbLinkConfigSelector);
    }

    /**
     * Reacts on clicking use-ajax link.
     *
     * @param {jQuery.Event} e
     *   The click event.
     */
    function gsActionAjax(e) {
      gsRegionActivate(e);
    }

    /**
     * Reacts on clicking non use-ajax link, fallback to AJAX.
     *
     * @param {jQuery.Event} e
     *   The click event.
     */
    function gsActionJs(e) {
      e.preventDefault();
      var $link = $(e.target);
      var $form;
      var $gsSection;

      gsRegionActivate(e);
      if (!$(_offcanvas).length) {
        $link.closest('.' + _gridstackAction).find('.' + _gsActionAjax).trigger('click');
      }
      else {
        $gsSection = $link.closest(_gsSection);
        // Clicking inside active section, show region quickly, no need an AJAX.
        if ($gsSection.length) {
          $form = $(_offcanvas + ' .layout-builder-configure-section');
          showActiveRegion($form, 'click');
        }
        // Clicking outside active section, trying to edit cross-section.
        else {
          $link.closest('.' + _gridstackAction).find('.' + _gsActionAjax).trigger('click');
        }
      }
    }

    /**
     * GridStack functions.
     *
     * @param {HTMLElement} elm
     *   The .gridstack__action HTML element.
     */
    function gsActionItem(elm) {
      var $elm = $(elm);
      var regionId = $elm.attr('data-gs-region');

      var addAttributes = function ($link) {
        var href = $link.attr('href');
        if (href) {
          var part = href.indexOf('?') > 0 ? '&gs=' : '?gs=';
          href = href + part + regionId;
          $link.attr('href', href).text(Drupal.t('Styles'));
        }
      };

      if ($lbConfig.length) {
        var $clone = $lbConfig.clone(true).addClass('visually-hidden layout-builder__link--add ' + _gsActionAjax);
        var $clone2 = $lbConfig.clone(false).addClass('layout-builder__link--add ' + _gsActionJs).removeClass('use-ajax').removeAttr('data-once data-dialog-type data-dialog-renderer');

        if (!$elm.find('.' + _gsActionAjax).length) {
          $elm.append($clone[0]);
        }

        var $link = $('.' + _gsActionAjax, elm);
        if ($link.length) {
          // Drupal.attachBehaviors($link[0]);
          addAttributes($link);

          if (!$elm.find('.' + _gsActionJs).length) {
            // Drupal.detachBehaviors($clone2[0]);
            $elm.append($clone2[0]);

            $link = $('.' + _gsActionJs, elm);
            addAttributes($link);
          }
        }

        $elm.on('mousedown.gsActionAjax', '.' + _gsActionAjax, gsActionAjax);
        $elm.on('click.gsActionJs', '.' + _gsActionJs, gsActionJs);
      }
      elm.classList.add(_gridstackAction + '--on');
    }

    /**
     * Provides inline styles.
     */
    function gsInlineStyles() {
      var stylesId;
      var curStyles;
      var _prefix = 'gridstack--style-';
      var config = $container.data('gsInlineStyles') || {};

      function initStyles() {
        var opts = {id: stylesId, prefix: _prefix};
        var gridStyle = GridStyle.init(opts, container);

        stylesId = gridStyle.id;
        curStyles = gridStyle.styles;
      }

      function updateStyles() {
        if (curStyles === null || typeof curStyles === 'undefined') {
          initStyles();
        }

        if (curStyles === null || typeof curStyles === 'undefined') {
          return;
        }

        initStyles();

        if (config) {
          var i = 0;
          _db.forEach(config, function (rule, selector) {
            GridStyle.insertCSSRule(curStyles, selector, rule, i);
            i++;
          });
        }
      }

      updateStyles();
    }

    gsInlineStyles();

    var actions = container.querySelectorAll('.' + _gridstackAction + ':not(.' + _gridstackAction + '--on)');
    if (actions.length) {
      _db.forEach(actions, gsActionItem);
    }

    container.classList.add(idContainer + '--on');
  }

  /**
   * Reacts on blur event.
   *
   * @param {jQuery.Event} e
   *   The blur event.
   */
  function gsLabelVariant(e) {
    var $input = $(e.target);
    var value = $input.val();
    var $form = $input.closest('.form--gridstack-variant');
    var $btn = $form.find('a.btn--editor-duplicate');

    if ($btn.length) {
      var href = $btn.attr('href');

      if (href.indexOf('label') > 0) {
        href = href.substring(0, href.indexOf('&label='));
      }

      href = href + '&label=' + Drupal.checkPlain(value);
      $btn.attr('href', href);
    }
  }

  /**
   * Attaches gridstack behavior to HTML element .gridstack.is-gs-lb.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.gridStackAdminLayout = {
    attach: function (context) {

      // Context is unreliable with AJAX contents.
      context = context instanceof HTMLDocument ? context : _doc;
      var containers = context.querySelectorAll(elContainer + ':not(.' + idContainer + '--on)');

      // The main gridstack container.
      if (containers.length) {
        _db.once(_db.forEach(containers, gsContainer, context));
      }

      $(_body).off('mousedown.' + idLbTopLink).on('mousedown.' + idLbTopLink, elLbTopLink, gsLbTopLink);
      $(_body).off('mousedown.' + idVariantEdit).on('mousedown.' + idVariantEdit, elVariantEdit, gsVariantEdit);
      $(_body).off('blur.' + idLabelVariant).on('blur.' + idLabelVariant, elLabelVariant, Drupal.debounce(gsLabelVariant, 200));

    },
    detach: function (context, settings, trigger) {
      if (trigger === 'unload') {
        $(_body).off('mousedown.' + idLbTopLink);
        $(_body).off('mousedown.' + idVariantEdit);
        $(_body).off('blur.' + idLabelVariant);
      }
    }
  };

})(jQuery, Drupal, dBlazy, GridStyle, this.document);
