/**
 * @file
 * Provides GridStack admin utilities for Layout Builder modal.
 *
 * Cheaper alt than full-blown class overrides via hook_element_plugin_alter
 * aside from the fact other modules might also race overriding that class.
 */

(function ($, Drupal, _db, _doc) {

  'use strict';

  var _offCanvas = 'drupal-off-canvas';
  var _mlSelection = 'gs-media-library-selection';
  var _lbHighligtedClass = 'is-layout-builder-highlighted';
  var _lbSectionSelector = '.layout-builder__section';
  var _dataGsSection = 'data-gs-section';
  var _gsRoot = '.is-gs-lb';
  var _gsRootActivated = _gsRoot + '.' + _lbHighligtedClass;
  var _dataGsRegion = 'data-gs-region';
  var _dataGsRegionSel = '[' + _dataGsRegion + ']';
  var _dataGsMediaStorage = 'data-gs-media-storage';
  var _mediaOverlay = '> .box__bg .media__overlay';
  var _palettes = '.form-wrapper--color-palettes';
  var _colorPicker = 'form-color--gs-color';
  var idColorPicker = 'gsColorPicker';
  var elColorPicker = '.' + _colorPicker;
  var _transparency = 'form-range--gs-color';
  var idTransparency = 'gsTransparency';
  var elTransparency = '.' + _transparency;
  var idAnimations = 'gsAnimations';
  var elAnimations = '.form-wrapper--animations';
  var elAnimationsSelector = '.form-select--gs-animation';
  var idMediaHeight = 'gsMediaHeight';
  var elMediaHeight = '.form-range--gs-lg-height';
  var idFieldsetCollapsible = 'gsFieldsetCollapsible';
  var elFieldsetCollapsible = 'fieldset.is-collapsible';
  var idPreset = 'gsPreset';
  var elPreset = '.form-wrapper[data-gs-preset-region]:not(input)';
  var idExtras = 'gsExtras';
  var elExtras = '.form-wrapper--extras:not(input)';
  var idVerticalMargin = 'gsVerticalMargin';
  var elVerticalMargin = '.form-item--vm';
  var observer = null;
  // var removed = false;
  var configMutationCanvas = {
    attributes: true,
    childList: true,
    subtree: true,
    attributeOldValue: true,
    characterData: false,
    attributeFilter: ['value']
  };

  /**
   * Returns GeidStack section.
   *
   * @return {Element}
   *   The .layout-builder__section HTML element.
   */
  function getSection() {
    var $gsRoot = $(_gsRootActivated);
    return $gsRoot.closest(_lbSectionSelector).attr(_dataGsSection, true);
  }

  /**
   * Return RGBA of a color given an alpha.
   *
   * @param {String} color
   *   The color value.
   * @param {String} alpha
   *   The alpha value.
   *
   * @return {String}
   *   The rgba color.
   */
  function makeRgba(color, alpha) {
    return 'rgba(' + parseInt(color.slice(-6, -4), 16) + ',' + parseInt(color.slice(-4, -2), 16) + ',' + parseInt(color.slice(-2), 16) + ',' + alpha + ')';
  }

  /**
   * Add the suffix due to not output via form API when combine with prefix.
   *
   * @param {Element} $elm
   *   The HTML element.
   */
  function addSuffix($elm) {
    var suffix = '<span class="field-suffix messages messages--error visually-hidden"></span>';
    var $suffix = $elm.next('.field-suffix');

    // @todo  && $elm.hasClass('form-range--gs-opacity')
    if (!$suffix.length) {
      $elm.after(suffix);
    }
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .form-color--gs HTML element.
   */
  function gsColorPicker(elm) {
    var $palettes = $(_palettes);

    $(elm).off('.gsColorPickerChange').on('change.gsColorPickerChange', function (e) {
      if ($palettes.length && $palettes.attr('open')) {
        $palettes.attr('open', false);
      }
    });

    $(elm).off('.' + idColorPicker).on('input.' + idColorPicker, function () {
      if ($palettes.length && !$palettes.attr('open')) {
        $palettes.attr('open', true);
      }
      var $input = $(this);
      $input.attr('value', $input.val());
    });

    elm.classList.add(idColorPicker + '--on');
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .form-color--gs HTML element.
   */
  function updateColorPicker(elm) {
    var $gsSection = getSection();
    var $input = $(elm);
    var color = $input.val();
    var property = $input.data(idColorPicker);
    var isBg = property === 'background-color';
    var region = $input.data('gsColorRegion');
    var target = $input.data('gsTargetSelector') || false;
    var $rgba = $('[data-gs-color-picker="rgba"][data-gs-color-region="' + region + '"]');
    var $alpha = $('.form-range--gs-alpha[data-gs-color-region="' + region + '"]');
    var alpha = $alpha.length ? $alpha.val() : -1;
    var rgba = isBg ? makeRgba(color, alpha) : color;
    var skip = color === '#000000';
    var visibleColor = skip ? '' : rgba;

    if ($gsSection.length) {
      var $selector = $gsSection.find('[data-region="' + region + '"]');
      if ($selector.length) {
        if (isBg && $selector.find(_mediaOverlay).length) {
          visibleColor = rgba;
          if ($selector.hasClass('is-gs-overlay')) {
            target = _mediaOverlay;
          }
          else {
            target = false;
            $selector.find(_mediaOverlay).css(property, '');
          }
        }

        if (!target) {
          $selector.css(property, visibleColor);
        }
        else {
          if (target === 'a' || target === 'a:hover') {
            target = target + ':not(.gridstack__action a)';
          }
          // @todo a:hover, of course.
          $selector.find(target).css(property, visibleColor);
        }
      }
    }

    if (isBg && $rgba.length) {
      $rgba.val(skip ? '' : rgba);
    }
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .form-wrapper--animations HTML element.
   */
  function gsAnimations(elm) {
    var $elm = $(elm);
    var $gsSection;
    var $animation = $(elAnimationsSelector, elm);
    var values = [];

    $animation.find('option').each(function (i, item) {
      var value = $(item).val();
      values.push(value);
    });

    $elm.off('.' + idAnimations).on('change.' + idAnimations, elAnimationsSelector, function () {
      $gsSection = getSection();

      var $input = $(this);
      var value = $input.val();
      var region = $input.data('gsAnimationRegion');
      var target = $input.data('gsTargetSelector') || false;

      if (target && $gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"] ' + target);
        if ($selector.length) {
          _db.forEach(values, function (val) {
            $selector.removeClass(val + ' animated');
          });

          if (value) {
            $selector.addClass(value + ' animated');
            $selector.attr('data-animation', value);
          }
          else {
            $selector.removeClass(value + ' animated');
            $selector.removeAttr('data-animation');
          }
        }
      }
    });

    $elm.off('.gsTextAnimation').on('blur.gsTextAnimation', '.form-text--gs', function () {
      $gsSection = getSection();

      var $input = $(this);
      var value = $input.val();
      var key = $input.data('gsAnimationKey');
      var region = $input.data('gsAnimationRegion');
      var target = $input.data('gsTargetSelector') || false;

      if (target && $gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"] ' + target);
        if ($selector.length) {
          if (value === '') {
            $selector.removeAttr('data-animation-' + key);
            $selector.css('animation-' + key, '');
          }
          else {
            $selector.attr('data-animation-' + key, value);
            $selector.addClass('animated');
            $selector.css('animation-' + key, value);
          }
        }
      }
    });
    elm.classList.add(idAnimations + '--on');
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .form-range--gs-alpha HTML element.
   */
  function gsTransparency(elm) {
    var $elm = $(elm);

    addSuffix($elm);

    $(elm).off('.' + idTransparency).on('change.' + idTransparency, function (e) {
      var $input = $(this);
      $input.attr('value', $input.val());
    });

    elm.classList.add(idTransparency + '--on');
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .form-range--gs-alpha HTML element.
   */
  function updateTransparency(elm) {
    var $input = $(elm);
    var type = $input.data(idColorPicker);
    var $gsSection;

    // Only applicable to BG for now.
    var alphaChannel = function () {
      $gsSection = getSection();

      var region = $input.data('gsColorRegion');
      var $color = $('[data-gs-color-picker="background-color"][data-gs-color-region="' + region + '"]');

      var alpha = $input.val();
      var color = $color.val();
      var $rgba = $('[data-gs-color-picker="rgba"][data-gs-color-region="' + region + '"]');
      var rgba = makeRgba(color, alpha);
      var $prefix = $input.prev('.field-prefix');
      var target = false;
      var property = 'background-color';
      var skip = color === '#000000';
      var visibleColor = skip ? 'transparent' : rgba;

      if ($gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"]');

        if ($selector.length) {
          if ($selector.find(_mediaOverlay).length) {
            if ($selector.hasClass('is-gs-overlay')) {
              target = _mediaOverlay;
            }
            else {
              target = false;
              // Remove pevious color, if switched.
              $selector.find(_mediaOverlay).css(property, '');
            }
          }

          if (!target) {
            $selector.css(property, visibleColor);
          }
          else {
            // @todo a:hover, of course.
            $selector.find(target).css(property, visibleColor);
          }
        }
      }

      // Store values regardless empty selectors, likely before adding images.
      if ($rgba.length) {
        $rgba.val(skip ? '' : rgba);
      }

      if ($prefix.length) {
        $prefix.text(alpha);
      }
    };

    var imageOpacity = function () {
      $gsSection = getSection();

      var region = $input.data('gsColorRegion');
      var alpha = $input.val();
      var target = $input.data('gsTargetSelector') || false;
      var $prefix = $input.prev('.field-prefix');

      if (target && $gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"] ' + target);
        if ($selector.length) {
          var $media = $selector.find('> .media__element');
          if ($media.length) {
            $selector.css('opacity', '');
            $selector = $media;
          }
          $selector.css('opacity', alpha);
        }
      }

      // Indicate values regardless empty, likely before adding images.
      if ($prefix.length) {
        $prefix.text(alpha);
      }
    };

    if (type === 'alpha') {
      alphaChannel();
    }
    else if (type === 'opacity') {
      imageOpacity();
    }
  }

  /**
   * GridStack functions.
   *
   * @param {Element} $input
   *   The input HTML element.
   * @param {Integer} value
   *   The default input value for an error.
   */
  function showError($input, value) {
    var $prefix = $input.prev('.field-prefix');
    var $suffix = $input.next('.field-suffix');

    if ($suffix.length) {
      $suffix.removeClass('visually-hidden').text(Drupal.t('Add image, or save the form first.'));
    }

    window.setTimeout(function () {
      $suffix.text('').assClass('visually-hidden');
    }, 6000);

    $input.val(value);

    if ($prefix.length) {
      $prefix.text(value);
    }
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .form-color--gs-height HTML element.
   */
  function gsMediaHeight(elm) {
    var $elm = $(elm);
    var $gsSection;

    addSuffix($elm);

    var updateMediaHeight = function (e) {
      $gsSection = getSection();
      if (!$gsSection.length) {
        return;
      }

      var $input = $(e.target);
      var region = $input.data('gsColorRegion');
      var value = $input.val();
      var $prefix = $input.prev('.field-prefix');
      var target = $input.data('gsTargetSelector') || '';
      var $selector = $gsSection.find('[data-region="' + region + '"]' + target);
      if ($selector.length) {
        value = value < 100 ? '' : value;
        $selector.css('height', value + 'px');
      }
      else {
        showError($input, 0);
      }

      if ($prefix.length) {
        $prefix.text(value);
      }
    };

    $(elm).off('.' + idMediaHeight).on('change.' + idMediaHeight, updateMediaHeight);
    elm.classList.add(idMediaHeight + '--on');
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The [data-gs-preset-region] HTML element.
   */
  function gsPreset(elm) {
    var $elm = $(elm);
    var values = [];
    var $gsSection;

    $elm.find('input').each(function (i, item) {
      var value = $(item).val();
      values.push(value);
    });

    var updatePreset = function (e) {
      $gsSection = getSection();
      if (!$gsSection.length) {
        return;
      }

      var input = e.target;
      var $input = $(input);
      var target = $input.data('gsTargetSelector') || '';
      var value = $input.val();
      var region = $input.attr('data-gs-preset-region');
      var $selector = $gsSection.find('[data-region="' + region + '"]' + target);

      if ($selector.length) {
        // @todo var $content = $selector.find('> .box__content');
        // @todo if ($content.length) {
        // @todo  $selector = $content;
        // @todo }
        _db.forEach(values, function (val) {
          $selector.removeClass(val);
        });

        if (value) {
          $selector.addClass(value);
        }
      }
    };

    $elm.off('.' + idPreset).on('click.' + idPreset, 'input', updatePreset);
    elm.classList.add(idPreset + '--on');
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .form-wrapper--extras HTML element.
   */
  function gsExtras(elm) {
    var $elm = $(elm);
    var $gsSection;

    $elm.off('.' + idExtras).on('click.' + idExtras, 'input', function (e) {
      $gsSection = getSection();
      if (!$gsSection.length) {
        return;
      }

      var input = e.target;
      var $input = $(input);
      var value = $input.val();
      var cssClass = input.dataset.gsExtrasRegionClass;
      var region = $input.attr('data-gs-extras-region');
      var $selector = $gsSection.find('[data-region="' + region + '"]');
      var $rgba = $('.form-color--gs-rgba[data-gs-color-region="' + region + '"]');
      var $bg = $('.form-color--gs-bg[data-gs-color-region="' + region + '"]');

      if ($selector.length) {
        var $content = $selector.find('> .box__content');
        if ($content.length) {
          $selector = $content;
        }

        var checked = $input.prop('checked');
        $selector[checked ? 'addClass' : 'removeClass'](cssClass);
        if (cssClass === 'is-gs-ete') {
          $selector.closest('.box')[checked ? 'addClass' : 'removeClass']('box--ete');
        }
        var $overlay = $selector.find(_mediaOverlay);
        switch (value) {
          case 'overlay':
            if ($overlay.length) {
              var previousColor = $rgba.val() || $bg.val();
              if (previousColor) {
                if (checked) {
                  $overlay.css('background-color', previousColor);
                }
                else {
                  $selector.css('background-color', previousColor);
                }
              }

              $overlay[checked ? 'show' : 'hide']();
            }
            break;

          default:
            break;
        }
      }
    });

    elm.classList.add(idExtras + '--on');
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .is-collapsible HTML element.
   */
  function gsFieldsetCollapsible(elm) {
    var $elm = $(elm);

    $elm.find('> legend > span').addClass('btn button');
    $elm.off('.' + idFieldsetCollapsible).on('click.' + idFieldsetCollapsible, 'legend', function (e) {
      var $fieldset = $(e.delegateTarget);
      $fieldset.toggleClass('is-collapsed');
    });
    elm.classList.add(idFieldsetCollapsible + '--on');
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The .form-item--vm HTML element.
   */
  function gsVerticalMargin(elm) {
    var $elm = $(elm);
    var values = [];
    var $gsRoot;

    $elm.children().each(function (i, item) {
      values.push(item.value);
    });

    $elm.off('.' + idVerticalMargin).on('change.' + idVerticalMargin, function (e) {
      $gsRoot = $(_gsRootActivated);
      if (!$gsRoot.length) {
        return;
      }

      var $select = $(e.target);
      var value = $select.val();

      _db.forEach(values, function (val) {
        $gsRoot.removeClass('vm-' + val);
      });
      $gsRoot.addClass('vm-' + value);
    });

    elm.classList.add(idVerticalMargin + '--on');
  }

  /**
   * GridStack functions.
   *
   * @param {Element} elm
   *   The target HTML element.
   * @param {String} mid
   *   The media ID, might be empty when deleted.
   */
  function updateMid(elm, mid) {
    var details = _db.closest(elm, _dataGsRegionSel);
    if (details) {
      var id = details.getAttribute(_dataGsRegion);
      var el = details.querySelector('[' + _dataGsMediaStorage + '="' + id + '"]');
      if (el) {
        el.value = mid;
        // Just to be clear.
        el.setAttribute('value', mid);
      }
    }
  }

  /**
   * GridStack functions.
   *
   * @param {Element} el
   *   The target HTML element.
   * @param {String} css
   *   The CSS class to apply.
   *
   * @return {Bool}
   *   True if contains the class.
   */
  function is(el, css) {
    return el && el.classList.contains(css);
  }

  /**
   * Callback function to execute when mutations are observed.
   *
   * @param {MutationRecord} mutations
   *   The recorded mutation list object.
   * @param {MutationObserver} observer
   *   The MutationObserver instance.
   *
   * @see https://dom.spec.whatwg.org/#mutationrecord
   * @see https://dom.spec.whatwg.org/#mutation-observers
   * @see https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver
   */
  function observeMutation(mutations, observer) {
    _db.forEach(mutations, function (mutation) {
      var target = mutation.target;
      if (target === null) {
        return;
      }

      // Skip useless mutations.
      if (target.nodeName && target.nodeName === 'SUMMARY') {
        return;
      }

      if (mutation.type === 'attributes') {
        if (mutation.attributeName === 'value') {
          // var name = target.getAttribute('name');
          // if (name && mutation.attributeName === 'disabled') {
          // removed = name.indexOf('media-library-remove') !== -1;
          // }
          if (is(target, _mlSelection)) {
            updateMid(target, target.value);
          }
          else if (is(target, _colorPicker)) {
            updateColorPicker(target);
          }
          else if (is(target, _transparency)) {
            updateTransparency(target);
          }
        }
      }

      if (mutation.type === 'childList') {
        var el = mutation.previousSibling;
        if (el && el.classList && el.classList.contains('media-library-item__remove')) {
          updateMid(target, '');
        }
      }
    });
  }

  /**
   * Attaches gridstack behavior to HTML element .gridstack.is-gs-lb.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.gridStackAdminModal = {
    attach: function (context) {

      // Context is unreliable with AJAX contents.
      context = context instanceof HTMLDocument ? context : _doc;

      function _select(el, id) {
        return context.querySelectorAll(el + ':not(.' + id + '--on)');
      }

      function _once(els, callback) {
        if (els.length) {
          _db.once(_db.forEach(els, callback, context));
        }
      }

      function _observe() {
        var elCanvas = _doc.getElementById(_offCanvas);
        observer = new MutationObserver(Drupal.debounce(observeMutation, 200));
        if (elCanvas) {
          observer.observe(elCanvas, configMutationCanvas);
        }
      }

      var pickers = _select(elColorPicker, idColorPicker);
      var trans = _select(elTransparency, idTransparency);
      var anims = _select(elAnimations, idAnimations);
      var medias = _select(elMediaHeight, idMediaHeight);
      var fiedsets = _select(elFieldsetCollapsible, idFieldsetCollapsible);
      var presets = _select(elPreset, idPreset);
      var extras = _select(elExtras, idExtras);
      var margins = _select(elVerticalMargin, idVerticalMargin);

      // Once for all, all for once.
      _once(pickers, gsColorPicker);
      _once(trans, gsTransparency);
      _once(anims, gsAnimations);
      _once(medias, gsMediaHeight);
      _once(fiedsets, gsFieldsetCollapsible);
      _once(presets, gsPreset);
      _once(extras, gsExtras);
      _once(margins, gsVerticalMargin);

      // Observe mutations.
      _observe();
    }
  };

})(jQuery, Drupal, dBlazy, this.document);
