/**
 * @file
 * Provides utilities for aspect ratio fixes.
 *
 * Blazy formatter Aspect ratio doesn't work out well with gapless grid.
 * _The reason: Aspect ratio is based on image dimensions. While Gridstack
 * wants images to fill in the entire box to look gapless ignoring the actual
 * dimensions. Layout design compactness is more important than image sizes.
 *
 * The warning was provided at TROUBLESHOOTING, however sometimes blazy contents
 * are also embedded outside gridstacks and on the contrary might need aspect
 * ratio to work correctly. To allow such re-usability for a blazy/ slick block,
 * hence we only concern those embedded inside gridstack.
 * We'll provide own solution to respect the gapless grid instead.
 *
 * This file is run before blazy.load.js so that blazy doesn't have to work.
 */

(function ($, Drupal) {

  'use strict';

  Drupal.gridstack = Drupal.gridstack || {};

  /**
   * Attaches behavior to HTML element identified by .gridstack--js.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.gridStackRatio = {
    attach: function (context) {

      var me = Drupal.gridstack;

      // Context is unreliable with AJAX contents like product variations, etc.
      context = me.context(context);

      var fixRatio = function (el) {
        var skip = $.closest(el, '.grid') !== null
        || $.closest(el, '.is-gs-contentless') !== null
        || el.classList.contains('is-gs-contentless')
        || el.parentNode.classList.contains('gridstack');

        if (skip) {
          return;
        }
        el.classList.add('b-noratio');
        el.className = el.className.replace(/media--ratio--(\d+)/g, '');
        el.classList.remove('media--ratio', 'media--ratio--fluid');
        el.removeAttribute('data-ratio');
        el.style.paddingBottom = '';
      };

      var checkRatio = function (el) {
        var items = el.querySelector('.media--ratio, .b-bg') === null ? [] : el.querySelectorAll('.media--ratio, .b-bg');
        if (items.length) {
          $.forEach(items, fixRatio, el);
        }
      };

      var elms = context.querySelectorAll('.gridstack--js:not(.is-gs-on)');
      if (elms.length > 0) {
        $.once($.forEach(elms, checkRatio, context));
      }
    }
  };

})(dBlazy, Drupal);
